var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import WS from 'ws';
import { AbstractWebSocketPeer } from 'ataraxia-ws-client';
import { AbstractTransport } from 'ataraxia/transport';
export class WebSocketServerTransport extends AbstractTransport {
    constructor(options) {
        super('ws-server');
        if (!options) {
            throw new Error('WebSocketServerTransport requires options, such as a port');
        }
        this.options = options;
    }
    start(options) {
        const _super = Object.create(null, {
            start: { get: () => super.start }
        });
        return __awaiter(this, void 0, void 0, function* () {
            const started = yield _super.start.call(this, options);
            if (!started)
                return false;
            this.server = new WS.Server(this.options);
            this.server.on('connection', socket => {
                const peer = new WebsocketServerPeer(this.network);
                this.addPeer(peer);
                peer.setSocket(socket);
            });
            this.server.on('error', err => {
                this.debug('Ignoring error:', err);
            });
            if (!this.options.server && !this.options.noServer) {
                // If not using an existing server wait until the server is started
                const server = this.server;
                return new Promise(resolve => server.on('listening', () => resolve(true)));
            }
            return true;
        });
    }
    stop() {
        const _super = Object.create(null, {
            stop: { get: () => super.stop }
        });
        return __awaiter(this, void 0, void 0, function* () {
            const stopped = yield _super.stop.call(this);
            if (!stopped)
                return false;
            if (this.server) {
                const server = this.server;
                yield new Promise(resolve => server.close(() => resolve()));
                this.server = undefined;
            }
            return true;
        });
    }
}
class WebsocketServerPeer extends AbstractWebSocketPeer {
    setSocket(socket) {
        super.setSocket(socket);
        this.negotiateAsServer();
    }
}
//# sourceMappingURL=WebSocketServerTransport.js.map